/*
* Robert Walker 31.12.17
* 2nd draft of mbed control software
* Declarations of Libraries
* Declarations of Pin Assignments
* mbed Nyquist limit is 22.814kHz
* motors to half step
* mbed XTAL clock frequency is 12MHz
*/

/* sample half step sequence
Motor1Out=duty_Cycle;
Motor2Out=0;
Motor3Out=0;
Motor4Out=duty_Cycle;
wait_ms(speed);
Motor1Out=duty_Cycle;
Motor2Out=duty_Cycle;
Motor3Out=0;
Motor4Out=0;
wait_ms(speed);
Motor1Out=0;
Motor2Out=duty_Cycle;
Motor3Out=duty_Cycle;
Motor4Out=0;
wait_ms(speed);
Motor1Out=0;
Motor2Out=0;
Motor3Out=duty_Cycle;
Motor4Out=duty_Cycle;
wait_ms(speed);
*/

#include "mbed.h" //include the mbed library in the compilation
 
DigitalIn Start(p5); //create a digital input called start. This will be an NO push button
DigitalIn Stop(p6); //create a digital input called stop. This will be an NC push button
DigitalIn SelectDirection(p7); //create a digital input called SelectDirection. This will be a SPST switch with pull up resistor. 1 is forward, 0 reverse
DigitalIn SelectUnipolar-Bi(p8); //create a digital input called SelectUnipolar-Bi. This will be a SPST switch with pull up resistor. 1 is bipolar, 0 unipolar
AnalogIn TempUnipolar(p15); //create an analogue input called TempUnipolar. This will be a Thermocouple through an instrumentation amplifier 
AnalogIn TempBipolar(p16); //create an analogue input called TempBipolar. This will be a Thermocouple through an instrumentation amplifier
AnalogIn POT1(p17); //create an analogue input connected through a buffer amplifier. this will measure the position of the servo
AnalogIn POT2(p18); //create an analogue input connected through a buffer amplifier
DigitalOut Uni/Bi_Multiplex(p27); //this will control which buffer set is connected to the PWM outputs from the mbed
DigitalOut Indicator3(p13); //5mm through hole LED on the PCB
DigitalOut Indicator4(p14); //5mm through hole LED on the PCB
DigitalOut Indicator5(LED1); //SMT LED on mbed PCB
DigitalOut Indicator6(LED2); //SMT LED on mbed PCB
DigitalOut Indicator7(LED3); //SMT LED on mbed PCB
DigitalOut Indicator8(LED4); //SMT LED on mbed PCB
PwmOut Motor1Out(p26); //Output to the stepper motors
PwmOut Motor2Out(p25); //Output to the stepper motors
PwmOut Motor3Out(p24); //Output to the stepper motors
PwmOut Motor4Out(p23); //Output to the stepper motors
PwmOut Indicator1(p22); //Output to an LED that mirrors the PWM signal to the motor
PwmOut Indicator2(p21); //Output to an LED that mirrors the PWM signal to the motor
int duty_Cycle=0; //create an integer variable called duty_cycle, which is used to set the synthetic PWM
int speed=0; //create an integer variable called speed, which sets the delay in the rotation of the stepper motor
int i=0; //create an integer variable called i, which sets up a sinusoidal signal
int steps=0; //count the number of steps in the travel of the stepper motor
int revolutions=0; //count the number of revolutions in the travel of the stepper motor
float vsin=0; //create a sinusoidal voltage to be outputted via pwm

/*
function prototypes
* int main(void)
* int unipolar_rotate(int)
* int bipolar_rotate(int)
* int emergency_handling(int)
*/

int main(void){
	for(;;){
		Motor1Out.period(0.001); //set the pwm period for Motor 1
		Motor2Out.period(0.001); //set the pwm period for Motor 1
		Motor3Out.period(0.001); //set the pwm period for Motor 1
		Motor4Out.period(0.001); //set the pwm period for Motor 1
		vsin=3.3*(0.5+(0.5*sin(i*3.14159); //create a sinusoidal waveform
		wait_ms(speed); //control frequency of sinusoid
		duty_Cycle=(POT1/3.3)*vsin; //calculate duty cycle of output
		speed=POT2; //select the speed of the motor
		if start == 1{ //software equivalent of DOL starter, with the special dispensation for PLC
			while(stop==1){ //while the stop button is not pressed
				while (revolutions<100) //if the number of stepper motor revolutions is less than 100, to simulate 
					Indicator2=1; //illuminate a indicator to show that the motor is rotating
					switch(SelectUnipolar-Bi){
						case: SelectUnipolar-Bi==0 //if the motor selection switch is low, then go to unipolar rotate
							unipolar_rotate(duty_Cycle) //go to the rotate unipolar motor sub, passing the pot value as the duty cycle
							revolutions=steps/200; //increment revolutions to ensure that
						end case //end case got to next one
						case: SelectUnipolar-Bi==1
							bipolar_rotate(duty_Cycle)
							revolutions=steps/48;
						end case
						Indicator1=duty_Cycle;
					
				}
			}
		}
		if stop==0{
			Indicator2=0;
		}
		if revolutions>100{
			Indicator2=0;
		}
	}
}

int unipolar_rotate(int duty_Cycle){	
	while(stop==1){
		if Uni/Bi_Multiplex==1{
			if SelectDirection==1;{
				Motor1Out=duty_Cycle;
				Motor2Out=0;
				Motor3Out=0;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				Motor1Out=duty_Cycle;
				Motor2Out=duty_Cycle;
				Motor3Out=0;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=duty_Cycle;
				Motor3Out=duty_Cycle;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=0;
				Motor3Out=duty_Cycle;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				steps=steps+4;
			}
			else{
				Motor1Out=duty_Cycle;
				Motor2Out=0;
				Motor3Out=0;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=0;
				Motor3Out=duty_Cycle;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=duty_Cycle;
				Motor3Out=duty_Cycle;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=duty_Cycle;
				Motor2Out=duty_Cycle;
				Motor3Out=0;
				Motor4Out=0;
				wait_ms(speed);
				steps=steps+4;
			}
		}
		else{ 
			bipolar_rotate
		}
	}
}


int bipolar_rotate(int duty_Cycle){
	while(stop==1){
		Uni/Bi_Multiplex==1
		if SelectUnipolar-Bi==0{
			if SelectDirection==1;{	
				Motor1Out=duty_Cycle;
				Motor2Out=0;
				Motor3Out=0;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				Motor1Out=duty_Cycle;
				Motor2Out=duty_Cycle;
				Motor3Out=0;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=duty_Cycle;
				Motor3Out=duty_Cycle;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=0;
				Motor3Out=duty_Cycle;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				steps=steps+4;
			}
			else{
				Motor1Out=duty_Cycle;
				Motor2Out=0;
				Motor3Out=0;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				Motor1Out=duty_Cycle;
				Motor2Out=duty_Cycle;
				Motor3Out=0;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=duty_Cycle;
				Motor3Out=duty_Cycle;
				Motor4Out=0;
				wait_ms(speed);
				Motor1Out=0;
				Motor2Out=0;
				Motor3Out=duty_Cycle;
				Motor4Out=duty_Cycle;
				wait_ms(speed);
				steps=steps+4;
			}
		}
	}
}

int emergency_handling(int tempBipolar int tempUnipolar){ //create a sub to manage emergencies within the system
	for(;;){ //create an infinite loop
		if tempUnipolar >= 100{ //if the temperature sensor for the unipolar bearing rises above 100, turn off motor
			stop=0;
		else 
			stop=1;
		}
		if tempBipolar >=100{ //if the temperature sensor for the unipolar bearing rises above 100, turn off motor
			stop=0;
		}
		else{
			stop=1;
		}
	}
}